<?php

namespace Lc\ShopBundle\Price\Services;

use Lc\ShopBundle\Context\OrderShopInterface;
use Lc\ShopBundle\Context\OrderShopPriceUtilsInterface;
use Lc\ShopBundle\Model\ReductionCart;

class OrderShopPriceUtils implements OrderShopPriceUtilsInterface
{
        use PriceUtilsTrait ;

        protected $orderProductPriceUtils ;

        public function __construct(OrderProductPriceUtils $orderProductPriceUtils)
        {
                $this->orderProductPriceUtils = $orderProductPriceUtils ;
        }

        //Inclus les ReductionCatalog des OrderProducts
        public function getTotalOrderProducts(OrderShopInterface $orderShop):float
        {
                // A tester calculer ce montant en faisant TotalOrderWithTax - TotalOrderTaxes

                $total = 0;
                foreach ($orderShop->getOrderProducts() as $orderProduct) {
                        $total += $this->orderProductPriceUtils->getTotalWithReduction($orderProduct);
                }
                return $total;
        }

        public function getTotalOrderProductsWithTax(OrderShopInterface $orderShop):float
        {
                $total = 0;
                foreach ($orderShop->getOrderProducts() as $orderProduct) {
                        $total += $this->orderProductPriceUtils->getTotalWithTaxAndReduction($orderProduct);
                }
                return $total;

        }


        public function getTotalOrderProductsTaxes(OrderShopInterface $orderShop):float {
        }

        public function getOrderProductsTaxesAsArray(OrderShopInterface $orderShop):array
        {
                $orderProductsTaxes = [];
                foreach ($orderShop->getOrderProducts() as $orderProduct) {

                        $idTaxRate = $orderProduct->getTaxRate()->getId() ;

                        if(!isset($orderProductsTaxes[$idTaxRate])) {
                                $orderProductsTaxes[$idTaxRate] = [
                                        'label' => $orderProduct->getTaxRate()->getValue() . '%',
                                        'totalOrderProducts' => 0,
                                        'totalTaxes' => 0,
                                ];
                        }

                        $orderProductsTaxes[$idTaxRate]['totalOrderProducts'] += $this->orderProductPriceUtils->getTotalWithReduction($orderProduct) * $this->getReductionsCoef($orderShop);
                        $orderProductsTaxes[$idTaxRate]['totalTaxes'] += $this->orderProductPriceUtils->getTotalTaxes($orderProduct) * $this->getReductionsCoef($orderShop) ;
                }

                return $orderProductsTaxes ;
        }

        private function getReductionsCoef(OrderShopInterface $orderShop) :float
        {
                return $this->getTotalOrderProducts($orderShop) / $this->getTotalOrderProductsWithReductions($orderShop);
        }

        private function getTaxRateAverage(OrderShopInterface $order):float
        {
                return $this->getTotalOrderProductsWithTax() / $this->getTotalOrderProducts();
        }

        public function getTotalOrderProductsWithReductions(OrderShopInterface $orderShop)
        {

        }


        public function getTotalWithTaxAndReductions(OrderShopInterface $orderShop)
        {

        }

        public function applyOrderProductsReductionCart(OrderShopInterface $order, $withTax = true)
        {

                foreach ($order->getOrderReductionCarts() as $orderReductionCart) {
                        if($orderReductionCart->getAppliedTo() == ReductionCart::APPLIED_TO_ORDER_PRODUCTS) {
                                if ($orderReductionCart->getUnit() == 'percent') {

                                        $priceWithTax = $this->applyReductionPercent(
                                                $this->getTotalOrderProductsWithTax($order),
                                                $orderReductionCart->getValue
                                        );
                                } else if ($orderReductionCart->getUnit() == 'amount') {
                                        if ($orderReductionCart->getBehaviorTaxRate() == 'tax-excluded') {
                                                $priceWithTax = $this->applyTax(
                                                        $this->applyReductionAmount(
                                                                $this->getTotalOrderProducts($order),
                                                                $orderReductionCart->getValue()
                                                        ),
                                                        $order->getTaxRateAverage()
                                                );
                                        } else if ($orderReductionCart->getBehaviorTaxRate() == 'tax-included') {
                                                $priceWithTax =
                                                        $this->applyReductionAmount(
                                                                $this->getTotalOrderProductsWithTax($order),
                                                                $orderReductionCart->getValue()
                                                        );

                                        }
                                }
                        }
                }

        }

}

